/*
 * Correction de l'exercice 2.3 utiliser un expression régulière 
 * pour obtenir les valeurs d'un chaîne contenant des données textuelles CSV
 * avec la possibilité d'avoir des ',' dans des valeurs délimitées par des '"'
 */

/**
 * découpe une chaîne de caractères correspondant à une ligne d'un fichier CSV
 * contenant des nombres en un tableau de valeurs numériques.
 * @param {string} csvString une chaîne correspondant à une ligne CSV
 * @returns le tableau des valeurs de cette ligne
 */
 function csvToStringArray(csvString) {
    let tokens = csvString.split(","); // on découpe a chaîne autours des ","
    for (let i = 0; i < tokens.length; i++) {
        tokens[i] = tokens[i].trim();  // on supprime les espaces en fin et début de chaîne
    }
    return tokens;
}

/**
 * découpe une chaîne de caractères correspondant à une ligne d'un fichier CSV
 * en un tableau de valeurs.
 * @param {string} csvString une chaîne correspondant à une ligne CSV
 * @returns le tableau des valeurs de cette ligne
 */
function csvToStringArray2(csvString) {
    let tokens = csvString.split(",");
    if (!csvString.includes('"')) {    // pas de '"', on utilise l'algorithme de l'exercice 2.2
        return csvToStringArray(csvString);
    }
    else {
        // la chaîne contient au moins une valeur délimitée par des "". 
        // decoupe la chaîne en sous chaînes, le caractère séparateur étant  ","
        let tokens = csvString.split(",");
        // Dans le cas où l'une des valeurs par des '"' contiendrait des  ','
        // le split initial a découpé la chaîne en plusieurs tokens, qu'il faut donc
        // regrouper avant d'enlever les "" délimitent la valeur
        let valeurs = [];  // le tableau résultat qui contiendra les valeurs
        let indexToken = 0;   // pour parcourir le tableau des 'tokens'
        let indexValeur = 0;  // pour parcourir le tableau des valeurs résultats
        while (indexToken < tokens.length) {
            if (! tokens[indexToken].includes('"')) {
                // le token correspond  à une valeur 'normale', il n'y a pas de "
                // au début. Il suffit simplement d'enlever les espaces en début
                // et les espaces en fin
                valeurs[indexValeur] = tokens[indexToken].trim();
            } else {
                // le token contient une valeur qui débute par "
                valeurs[indexValeur] = tokens[indexToken];
                if (valeurs[indexValeur].indexOf('"') === valeurs[indexValeur].lastIndexOf('"')) {
                    // la valeur ne contient qu'un seul ". Pour reconstruire la valeur complète
                    // il va faloir ajouter à la valeur les tokens suivants (en ré-insérant
                    // les ',' qui ont servi à faire le split), jusqu'à ce que l'on arrive
                    // au token contenant le " fermant.
                    let fermetureChaine = false;
                    do {
                        indexToken++;
                        // on rajoute le token à la valeur
                        valeurs[indexValeur] = valeurs[indexValeur] + "," + tokens[indexToken];
                        // on teste si le token indexToken est celui qui termine la valeur, c'est 
                        // à dire si il contient "
                        fermetureChaine = tokens[indexToken].includes('"');

                    } while (indexToken <= tokens.length && !fermetureChaine)
                }
                // dans valeurs se trouve la chaîne complète reconsituée
                // on enlève les espaces en début et fin
                valeurs[indexValeur] = valeurs[indexValeur].trim();
                // on enlève les ""
                valeurs[indexValeur] = valeurs[indexValeur].substring(1, valeurs[indexValeur].length - 1);
                // on enlève à nouveau les espaces en début et fin
                valeurs[indexValeur] = valeurs[indexValeur].trim();
            }
            indexValeur++;
            indexToken++;
        }
        return valeurs;
    }

}


// les différentes chaînes à tester
let testStr = [
    'valeur1, valeur 2, valeur 3 et 4 , valeur5',
    'Valeur1 , Valeur2,  Valeur 3,  "valeur avec une ,  dedans",  une dernière valeur',
    '"  Valeur entre quotes "   , Valeur1 , Valeur2,  Valeur avec plusieurs mots,  "   valeur contenant , et , "     ,  autre valeur sans quotes  , "valeur entre quotes suivie d\'une valeur vide",, "une dernière après des ,," '
];


for (let str of testStr) {
    let res = csvToStringArray2(str);
    console.log('la chaine à traiter\n' + str);
    console.log('\nLe tableau résultat\n');
    console.table(res);
}